import nonebot
from pathlib import Path
try:
    import ujson as json
except ModuleNotFoundError:
    import json

DRAW_PATH = str(Path("data/draw_card/").absolute()) + '/'

# 方舟概率
PRTS_SIX_P = 0.02
PRTS_FIVE_P = 0.08
PRTS_FOUR_P = 0.48
PRTS_THREE_P = 0.42

# 原神概率
GENSHIN_FIVE_P = 0.006
GENSHIN_FOUR_P = 0.051
GENSHIN_G_FOUR_P = 0.13
GENSHIN_G_FIVE_P = 0.016
I72_ADD = 0.0585

# 赛马娘概率
PRETTY_THREE = 0.03
PRETTY_TWO = 0.18
PRETTY_ONE = 0.79

path_dict = {
    'genshin': '原神',
    'prts': '明日方舟',
    'pretty': '赛马娘',
}


_draw_config = Path() / "data" / "draw_card" / "draw_card_config" / "draw_card_config.json"


driver: nonebot.Driver = nonebot.get_driver()


@driver.on_startup
def check_config():
    global PRTS_SIX_P, PRTS_FOUR_P, PRTS_FIVE_P, PRTS_THREE_P, GENSHIN_G_FIVE_P,\
        GENSHIN_G_FOUR_P, GENSHIN_FOUR_P, GENSHIN_FIVE_P, I72_ADD, path_dict, PRETTY_THREE, PRETTY_ONE, PRETTY_TWO
    if _draw_config.exists():
        data = json.load(open(_draw_config, 'r', encoding='utf8'))
        PRTS_SIX_P = float(data['prts']['six'])
        PRTS_FIVE_P = float(data['prts']['five'])
        PRTS_FOUR_P = float(data['prts']['four'])
        PRTS_THREE_P = float(data['prts']['three'])

        GENSHIN_FIVE_P = float(data['genshin']['five_char'])
        GENSHIN_FOUR_P = float(data['genshin']['four_char'])
        GENSHIN_G_FIVE_P = float(data['genshin']['five_weapon'])
        GENSHIN_G_FOUR_P = float(data['genshin']['four_weapon'])
        I72_ADD = float(data['genshin']['72_add'])

        PRETTY_THREE = float(data['pretty']['three'])
        PRETTY_TWO = float(data['pretty']['two'])
        PRETTY_ONE = float(data['pretty']['one'])

        path_dict = data['path_dict']
    else:
        _draw_config.parent.mkdir(parents=True, exist_ok=True)
        config_dict = {
            'path_dict': {
                'genshin': '原神',
                'prts': '明日方舟',
                'pretty': '赛马娘',
            },

            'prts': {
                'six': 0.02,
                'five': 0.08,
                'four': 0.48,
                'three': 0.42,
            },

            'genshin': {
                'five_char': 0.006,
                'four_char': 0.051,
                'five_weapon': 0.13,
                'four_weapon': 0.016,
                '72_add': 0.0585,
            },

            'pretty': {
                'three': 0.03,
                'two': 0.18,
                'one': 0.79,
            }
        }
        json.dump(config_dict, open(_draw_config, 'w', encoding='utf8'), indent=4, ensure_ascii=False)












import ujson as json
import os
from nonebot.adapters.cqhttp import MessageSegment
import nonebot
import random
from .config import PRTS_FIVE_P, PRTS_FOUR_P, PRTS_SIX_P, DRAW_PATH
from .update_game_info import update_info
from .util import generate_img, init_star_rst, max_card
from .init_card_pool import init_game_pool
from pathlib import Path
from .announcement import PrtsAnnouncement

driver: nonebot.Driver = nonebot.get_driver()

up_char_file = Path() / "data" / "draw_card" / "draw_card_up" / "prts_up_char.json"

up_char_dict = {}
prts_dict = {}
FIVE_LIST = []
FOUR_LIST = []
SIX_LIST = []
THREE_LIST = []
_current_pool_title = ''


async def prts_draw(count: int = 300):
    cnlist = ['★★★★★★', '★★★★★', '★★★★', '★★★']
    operator_list, operator_dict, six_list, star_list, six_olist = _format_card_information(count)
    up_list = []
    if _current_pool_title:
        for key in up_char_dict.keys():
            for value in up_char_dict[key].values():
                for i in value:
                    up_list.append(i)
    rst = init_star_rst(star_list, cnlist, six_list, six_olist, up_list)
    if count > 90:
        operator_list = set(operator_list)
    pool_info = "当前up池: " if _current_pool_title else ""
    return pool_info + _current_pool_title + MessageSegment.image("base64://" + await generate_img(operator_list, 'prts', star_list, [SIX_LIST, FIVE_LIST, FOUR_LIST]))\
           + '\n' + rst[:-1] + '\n' + max_card(operator_dict)


async def update_prts_info():
    global prts_dict, SIX_LIST, FOUR_LIST, FIVE_LIST, THREE_LIST
    url = 'https://wiki.biligame.com/arknights/干员数据表'
    data, code = await update_info(url, 'prts', ['头像', '名称', '阵营', '星级', '性别', '是否感染', '初始生命', '初始防御',
                                                 '初始法抗', '再部署', '部署费用', '阻挡数', '攻击速度', '标签'])
    if code == 200:
        prts_dict = data
        SIX_LIST, FIVE_LIST, FOUR_LIST, THREE_LIST = init_game_pool('prts', data)


@driver.on_startup
async def init_data():
    global prts_dict, SIX_LIST, FIVE_LIST, FOUR_LIST, THREE_LIST
    if not os.path.exists(DRAW_PATH + 'prts.json'):
        await update_prts_info()
    else:
        with open(DRAW_PATH + 'prts.json', 'r', encoding='utf8') as f:
            prts_dict = json.load(f)
        SIX_LIST, FIVE_LIST, FOUR_LIST, THREE_LIST = init_game_pool('prts', prts_dict)
    await _init_up_char()


# 抽取干员
def _get_operator_card():
    global up_char_dict
    rand = random.random()
    if rand <= PRTS_SIX_P:
        operator = _get_up_operator('6')
        if operator:
            return operator, 0
        return random.choice(SIX_LIST), 0
    elif rand <= PRTS_FIVE_P:
        operator = _get_up_operator('5')
        if operator:
            return operator, 1
        return random.choice(FIVE_LIST), 1
    elif rand <= PRTS_FOUR_P:
        operator = _get_up_operator('4')
        if operator:
            return operator, 2
        return random.choice(FOUR_LIST), 2
    else:
        return random.choice(THREE_LIST), 3


# 获得up干员
def _get_up_operator(star: str):
    # print(up_char_dict)
    if _current_pool_title and up_char_dict and up_char_dict[star]:
        rand = random.random()
        # print(f'rand: {rand}')
        for probability in up_char_dict[star].keys():
            if probability < 1:
                if rand < probability:
                    return random.choice(up_char_dict[star][probability])
            else:
                if rand < 1 / float(len(SIX_LIST) + len(up_char_dict[star][probability])) * probability:
                    return random.choice(up_char_dict[star][probability])
    return None


# 整理抽卡数据
def _format_card_information(count: int):
    operator_list = []  # 抽取的干员列表
    operator_dict = {}  # 抽取各干员次数
    star_list = [0, 0, 0, 0]    # 各个星级次数
    six_list = []   # 六星干员列表
    six_olist = []  # 六星干员获取位置
    for i in range(count):
        name, code = _get_operator_card()
        star_list[code] += 1
        if code == 0:
            six_list.append(name)
            six_olist.append(i)
        try:
            operator_dict[name] += 1
        except KeyError:
            operator_dict[name] = 1
        operator_list.append(name)
    return operator_list, operator_dict, six_list, star_list, six_olist


# 获取up干员和概率
async def _init_up_char():
    global up_char_dict, SIX_LIST, FIVE_LIST, FOUR_LIST, _current_pool_title
    up_char_dict = await PrtsAnnouncement().update_up_char()
    print(up_char_dict)
    if _current_pool_title == up_char_dict['title']:
        return
    _current_pool_title = up_char_dict['title']
    up_char_dict = up_char_dict['up_char']
    print('成功获取明日方舟当前up信息...')
    average_dict = {'6': {}, '5': {}, '4': {}}
    for star in up_char_dict.keys():
        for key in up_char_dict[star].keys():
            if average_dict[star].get(up_char_dict[star][key]):
                average_dict[star][up_char_dict[star][key]].append(key)
            else:
                average_dict[star][up_char_dict[star][key]] = [key]
    up_char_dict = {'6': {}, '5': {}, '4': {}}
    for star in average_dict.keys():
        for key in average_dict[star].keys():
            try:
                if not up_char_dict[star].get(float(key) / 100):
                    up_char_dict[star][float(key) / 100] = average_dict[star][key]
            except ValueError:
                up_char_dict[star][int(key[1:])] = average_dict[star][key]


async def reload_pool():
    await _init_up_char()









#coding:utf-8
import aiohttp
from .config import DRAW_PATH
from asyncio.exceptions import TimeoutError
from bs4 import BeautifulSoup
from .util import download_img
from urllib.parse import unquote
import bs4
import re
try:
    import ujson as json
except ModuleNotFoundError:
    import json


headers = {'User-Agent': '"Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; TencentTraveler 4.0)"'}


async def update_info(url: str, game_name: str, info_list: list = None) -> 'dict, int':
    try:
        with open(DRAW_PATH + f'{game_name}.json', 'r', encoding='utf8') as f:
            data = json.load(f)
    except (ValueError, FileNotFoundError):
        data = {}
    try:
        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.get(url, timeout=7) as response:
                soup = BeautifulSoup(await response.text(), 'lxml')
                max_count = 0
                _tbody = None
                for tbody in soup.find_all('tbody'):
                    if len(tbody.find_all('tr')) > max_count:
                        _tbody = tbody
                        max_count = len(tbody.find_all('tr'))
                trs = _tbody.find_all('tr')
                att_dict = {'头像': 0, '名称': 1}
                index = 2
                for th in trs[0].find_all('th')[2:]:
                    text = th.text
                    if text[-1] == '\n':
                        text = text[:-1]
                    att_dict[text] = index
                    index += 1
                for tr in trs[1:]:
                    member_dict = {}
                    k_name = ''
                    tds = tr.find_all('td')
                    if not info_list:
                        info_list = att_dict.keys()
                    for key in info_list:
                        attr = ''
                        if key.find('.') != -1:
                            key = key.split('.')
                            attr = key[-1]
                            key = key[0]
                        td = tds[att_dict[key]]
                        last_tag = unquote(_find_last_tag(td, attr), 'utf-8')
                        if game_name.find('pretty') == -1 and last_tag.find('http') == -1:
                            last_tag = last_tag.split('.')[0]
                        if key == '名称':
                            k_name = last_tag
                        member_dict[key] = last_tag
                        if game_name == 'pretty' and key == '初始星级':
                            member_dict['初始星级'] = len(td.find_all('img'))
                    avatar_img = await _modify_avatar_url(session, game_name, member_dict["名称"])
                    if avatar_img:
                        member_dict['头像'] = avatar_img
                    name = member_dict['名称']
                    if game_name == 'pretty_card':
                        name = member_dict['中文名']
                    await download_img(member_dict['头像'], game_name, name)
                    if k_name:
                        data[k_name] = member_dict
                    print(f'{k_name} is update...')
            data = await _last_check(data, game_name, session)
    except TimeoutError:
        return {}, 999
    with open(DRAW_PATH + f'{game_name}.json', 'w', encoding='utf8') as wf:
        wf.write(json.dumps(data, ensure_ascii=False, indent=4))
    return data, 200


def _find_last_tag(element: bs4.element.Tag, attr: str) -> str:
    last_tag = []
    for des in element.descendants:
        last_tag.append(des)
    if len(last_tag) == 1 and last_tag[0] == '\n':
        last_tag = ''
    elif last_tag[-1] == '\n':
        last_tag = last_tag[-2]
    else:
        last_tag = last_tag[-1]
    if attr and str(last_tag):
        last_tag = last_tag[attr]
    elif str(last_tag).find('<img') != -1:
        if last_tag.get('srcset'):
            last_tag = str(last_tag.get('srcset')).strip().split(' ')[-2].strip()
        else:
            last_tag = last_tag['src']
    else:
        last_tag = str(last_tag)
    if str(last_tag) and str(last_tag)[-1] == '\n':
        last_tag = str(last_tag)[:-1]
    return last_tag


# 获取大图（小图快爬）
async def _modify_avatar_url(session: aiohttp.ClientSession, game_name: str, char_name: str):
    if game_name == 'prts':
        async with session.get(f'https://wiki.biligame.com/arknights/{char_name}', timeout=7) as res:
            soup = BeautifulSoup(await res.text(), 'lxml')
            try:
                img_url = str(soup.find('img', {'class': 'img-bg'})['srcset']).split(' ')[-2]
            except KeyError:
                img_url = str(soup.find('img', {'class': 'img-bg'})['src'])
            return img_url
    if game_name == 'genshin':
        return None
    if game_name == 'pretty_card':
        # print(f'this is {char_name}')
        async with session.get(f'https://wiki.biligame.com/umamusume/{char_name}', timeout=7) as res:
            soup = BeautifulSoup(await res.text(), 'lxml')
            img_url = soup.find('div', {'class': 'support_card-left'}).find('div').find('img').get('src')
            return img_url


# 数据最后处理（是否需要额外数据）
async def _last_check(data: dict, game_name: str, session: aiohttp.ClientSession):
    if game_name == 'prts':
        for key in data.keys():
            async with session.get(f'https://wiki.biligame.com/arknights/{key}', timeout=7) as res:
                soup = BeautifulSoup(await res.text(), 'lxml')
                obtain = str(soup.find('table', {'class': 'wikitable'}).find('tbody').find_all('td')[-1])
                obtain = re.search(r'<td.*?>([\s\S]*)</.*?', obtain).group(1)
                obtain = obtain[:-1] if obtain[-1] == '\n' else obtain
                if obtain.find('<br/>'):
                    obtain = obtain.split('<br/>')
                elif obtain.find('<br>'):
                    obtain = obtain.split('<br>')
                data[key]['获取途径'] = obtain
    if game_name == 'pretty':
        for keys in data.keys():
            for key in data[keys].keys():
                # print(f'key --> {data[keys][key]}')
                r = re.search(r'.*?40px-(.*)图标.png', str(data[keys][key]))
                if r:
                    data[keys][key] = r.group(1)
    return data




# ul = soup.find('div', {'class': 'savelist_bot'}).find('ul')







def init_game_pool(game: str, data: dict):
    if game == 'prts':
        SIX_LIST = [data[name]['名称'] for name in data if data[name]['星级'] == '6' and '干员寻访' in data[name]['获取途径']]
        FIVE_LIST = [data[name]['名称'] for name in data if data[name]['星级'] == '5' and '干员寻访' in data[name]['获取途径']]
        FOUR_LIST = [data[name]['名称'] for name in data if data[name]['星级'] == '4' and '干员寻访' in data[name]['获取途径']]
        THREE_LIST = [data[name]['名称'] for name in data if data[name]['星级'] == '3' and '干员寻访' in data[name]['获取途径']]
        return SIX_LIST, FIVE_LIST, FOUR_LIST, THREE_LIST




from nonebot import on_command, on_regex, require, on_keyword
from nonebot.adapters.cqhttp import Bot, MessageEvent
from nonebot.permission import SUPERUSER
from nonebot.typing import T_State
from .genshin_handle import genshin_draw, update_genshin_info, reset_count
from .prts_handle import update_prts_info, prts_draw, reload_pool
from .pretty_handle import update_pretty_info, pretty_draw
from .update_game_info import update_info
from .util import is_number
import re

scheduler = require('nonebot_plugin_apscheduler').scheduler

prts = on_regex(r'.*?方舟[1-9|一][0-9]{0,2}[抽|井]', priority=5, block=True)
prts_update = on_command('更新明日方舟信息', aliases={'更新方舟信息'}, permission=SUPERUSER, priority=1, block=True)
prts_reload = on_keyword({'重载方舟卡池'}, priority=5, block=True)

genshin = on_regex('.*?原神[1-9|一][0-9]{0,2}[抽|井]', priority=5, block=True)
reset_genshin = on_command('重置原神抽卡次数', priority=5, block=True)
genshin_update = on_command('更新原神信息', permission=SUPERUSER, priority=1, block=True)

pretty = on_regex('.*?马娘卡?[1-9|一][0-9]{0,2}[抽|井]', priority=5, block=True)
pretty_update = on_command('更新赛马娘信息', aliases={'更新马娘信息'}, permission=SUPERUSER, priority=1, block=True)


# pretty = on_regex('赛马娘?[1-9|一][0-9]{0,2}[抽|井]', priority=5, block=True)
# pretty_update = on_command('更新赛马娘信息', aliases={'更新马娘信息'}, permission=SUPERUSER, priority=1, block=True)


@prts.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    msg = str(event.get_message()).strip()
    if msg in ['方舟一井', '方舟1井']:
        num = 300
    else:
        rmsg = re.search(r'.*?方舟(.*)抽', msg)
        if rmsg and is_number(rmsg.group(1)):
            try:
                num = int(rmsg.group(1))
            except ValueError:
                await prts.finish('必！须！是！数！字！', at_sender=True)
            if num > 300:
                await prts.finish('一井都满不足不了你嘛！快爬开！', at_sender=True)
            if num < 1:
                await prts.finish('虚空抽卡？？？', at_sender=True)
        else:
            return
    print(num)
    await prts.send(await prts_draw(num), at_sender=True)


@prts_update.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    await update_prts_info()
    await reload_pool()
    await prts_update.finish('更新完成！')


@prts_reload.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    await reload_pool()
    await prts_reload.finish('重载完成！')


@genshin.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    msg = str(event.get_message()).strip()
    if msg in ['原神一井', '原神1井']:
        num = 180
    else:
        rmsg = re.search(r'.*?原神(.*)抽', msg)
        if rmsg and is_number(rmsg.group(1)):
            try:
                num = int(rmsg.group(1))
            except ValueError:
                await genshin.finish('必！须！是！数！字！', at_sender=True)
            if num > 300:
                await genshin.finish('一井都满不足不了你嘛！快爬开！', at_sender=True)
            if num < 1:
                await genshin.finish('虚空抽卡？？？', at_sender=True)
        else:
            return
    await genshin.send(await genshin_draw(event.user_id, num), at_sender=True)


@reset_genshin.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    reset_count(event.user_id)
    await reset_genshin.send('重置了原神抽卡次数', at_sender=True)


@pretty.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    msg = str(event.get_message()).strip()
    if msg in ['赛马娘一井', '赛马娘1井', '马娘一井', '马娘1井', '赛马娘卡一井', '赛马娘卡1井', '马娘卡一井', '马娘卡1井']:
        num = 200
        if msg.find("卡") == -1:
            pool_name = 'horse'
        else:
            pool_name = 'card'
    else:
        rmsg = re.search(r'.*?马娘(.*)抽', msg)
        if rmsg:
            num = rmsg.group(1)
            if num[0] == '卡':
                num = num[1:]
                pool_name = 'card'
            else:
                pool_name = 'horse'
            if is_number(num):
                try:
                    num = int(num)
                except ValueError:
                    await genshin.finish('必！须！是！数！字！', at_sender=True)
                if num > 200:
                    await genshin.finish('一井都满不足不了你嘛！快爬开！', at_sender=True)
                if num < 1:
                    await genshin.finish('虚空抽卡？？？', at_sender=True)
            else:
                return
    await pretty.send(await pretty_draw(num, pool_name), at_sender=True)


@genshin_update.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    await update_genshin_info()
    await genshin_update.finish('更新完成！')


@pretty_update.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    await update_pretty_info()
    await genshin_update.finish('更新完成！')


# 更新资源
@scheduler.scheduled_job(
    'cron',
    hour=4,
    minute=1,
)
async def _():
    try:
        await update_prts_info()
    except Exception as e:
        pass
    try:
        await update_genshin_info()
    except Exception as e:
        pass
    try:
        await update_pretty_info()
    except Exception as e:
        pass


# 每天四点重载up卡池
@scheduler.scheduled_job(
    'cron',
    hour=4,
    minute=1,
)
async def _():
    await reload_pool()





import ujson as json
import os
import nonebot
from nonebot.adapters.cqhttp import MessageSegment
from .update_game_info import update_info
from .util import download_img, init_star_rst, generate_img, max_card
import random
from .config import PRETTY_THREE, PRETTY_TWO, DRAW_PATH

driver: nonebot.Driver = nonebot.get_driver()

pretty_char_dict = {}
pretty_card_dict = {}
THREE_LIST = []
SSR_LIST = []
TWO_LIST = []
SR_LIST = []
ONE_LIST = []
R_LIST = []


async def pretty_draw(count: int, pool_name):
    if pool_name == 'card':
        cnlist = ['SSR', 'SR', 'R']
        A_LIST = SSR_LIST
        B_LIST = SR_LIST
        C_LIST = R_LIST
    else:
        cnlist = ['★★★', '★★', '★']
        A_LIST = THREE_LIST
        B_LIST = TWO_LIST
        C_LIST = ONE_LIST
    obj_list, obj_dict, three_list, star_list, three_olist = _format_card_information(count, pool_name)
    rst = init_star_rst(star_list, cnlist, three_list, three_olist)
    if count > 100:
        obj_list = set(obj_list)
    return MessageSegment.image("base64://" + await generate_img(obj_list, 'pretty', star_list, [A_LIST, B_LIST, C_LIST])) \
           + '\n' + rst[:-1] + '\n' + max_card(obj_dict)


async def update_pretty_info():
    global pretty_char_dict, pretty_card_dict, TWO_LIST, THREE_LIST, ONE_LIST, SSR_LIST, SR_LIST, R_LIST
    url = 'https://wiki.biligame.com/umamusume/赛马娘图鉴'
    data, code = await update_info(url, 'pretty')
    if code == 200:
        pretty_char_dict = data
        THREE_LIST = [pretty_char_dict[char]['名称'] for char in pretty_char_dict.keys() if
                      pretty_char_dict[char]['初始星级'] == 3]
        TWO_LIST = [pretty_char_dict[char]['名称'] for char in pretty_char_dict.keys() if
                    pretty_char_dict[char]['初始星级'] == 2]
        ONE_LIST = [pretty_char_dict[char]['名称'] for char in pretty_char_dict.keys() if
                    pretty_char_dict[char]['初始星级'] == 1]
    url = 'https://wiki.biligame.com/umamusume/支援卡图鉴'
    data, code = await update_info(url, 'pretty_card')
    if code == 200:
        pretty_card_dict = data
        SSR_LIST = [pretty_card_dict[card]['中文名'] for card in pretty_card_dict.keys() if
                    pretty_card_dict[card]['稀有度'] == "SSR"]
        SR_LIST = [pretty_card_dict[card]['中文名'] for card in pretty_card_dict.keys() if
                   pretty_card_dict[card]['稀有度'] == "SR"]
        R_LIST = [pretty_card_dict[card]['中文名'] for card in pretty_card_dict.keys() if
                  pretty_card_dict[card]['稀有度'] == "R"]


@driver.on_startup
async def init_data():
    global pretty_char_dict, pretty_card_dict, TWO_LIST, THREE_LIST, ONE_LIST, SSR_LIST, SR_LIST, R_LIST
    if not os.path.exists(DRAW_PATH + 'pretty.json') or not os.path.exists(DRAW_PATH + 'pretty_card.json'):
        await update_pretty_info()
    else:
        with open(DRAW_PATH + 'pretty.json', 'r', encoding='utf8') as f:
            pretty_char_dict = json.load(f)
        with open(DRAW_PATH + 'pretty_card.json', 'r', encoding='utf8') as f:
            pretty_card_dict = json.load(f)
        THREE_LIST = [pretty_char_dict[char]['名称'] for char in pretty_char_dict.keys() if
                      pretty_char_dict[char]['初始星级'] == 3]
        TWO_LIST = [pretty_char_dict[char]['名称'] for char in pretty_char_dict.keys() if
                    pretty_char_dict[char]['初始星级'] == 2]
        ONE_LIST = [pretty_char_dict[char]['名称'] for char in pretty_char_dict.keys() if
                    pretty_char_dict[char]['初始星级'] == 1]
        SSR_LIST = [pretty_card_dict[card]['中文名'] for card in pretty_card_dict.keys() if
                    pretty_card_dict[card]['稀有度'] == "SSR"]
        SR_LIST = [pretty_card_dict[card]['中文名'] for card in pretty_card_dict.keys() if
                   pretty_card_dict[card]['稀有度'] == "SR"]
        R_LIST = [pretty_card_dict[card]['中文名'] for card in pretty_card_dict.keys() if
                  pretty_card_dict[card]['稀有度'] == "R"]
    for icon_url in [
        'https://patchwiki.biligame.com/images/umamusume/thumb/0/06/q23szwkbtd7pfkqrk3wcjlxxt9z595o.png'
        '/40px-SSR.png',
        'https://patchwiki.biligame.com/images/umamusume/thumb/3/3b/d1jmpwrsk4irkes1gdvoos4ic6rmuht.png'
        '/40px-SR.png',
        'https://patchwiki.biligame.com/images/umamusume/thumb/f/f7/afqs7h4snmvovsrlifq5ib8vlpu2wvk.png'
        '/40px-R.png']:
        await download_img(icon_url, 'pretty', icon_url.split('-')[-1][:-4])


# 抽取卡池
def _get_pretty_card(itype):
    rand = random.random()
    # 普通抽
    if rand < PRETTY_THREE:
        if itype == 'card':
            return random.choice(SSR_LIST), 0
        return random.choice(THREE_LIST), 0
    elif rand < PRETTY_TWO:
        if itype == 'card':
            return random.choice(SR_LIST), 1
        return random.choice(TWO_LIST), 1
    else:
        if itype == 'card':
            return random.choice(R_LIST), 2
        return random.choice(ONE_LIST), 2


# 整理数据
def _format_card_information(count: int, pool_name: str):
    three_list = []
    three_olist = []
    obj_list = []
    obj_dict = {}
    star_list = [0, 0, 0]
    for i in range(count):
        name, code = _get_pretty_card(pool_name)
        star_list[code] += 1
        if code == 0:
            three_list.append(name)
            three_olist.append(i)
        try:
            obj_dict[name] += 1
        except KeyError:
            obj_dict[name] = 1
        obj_list.append(name)
    return obj_list, obj_dict, three_list, star_list, three_olist













import os
import aiohttp
import aiofiles
from asyncio.exceptions import TimeoutError
from aiohttp.client_exceptions import InvalidURL
from typing import List, Union, Set
import asyncio
from pathlib import Path
from .config import path_dict, DRAW_PATH
import nonebot
import pypinyin
from .create_img import CreateImg
try:
    import ujson as json
except ModuleNotFoundError:
    import json


driver: nonebot.Driver = nonebot.get_driver()


headers = {'User-Agent': '"Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1; TencentTraveler 4.0)"'}


async def download_img(url: str, path: str, name: str) -> bool:
    path = path.split('_')[0]
    codename = cn2py(name)
    # if not _p.exists():
    #     _p.mkdir(parents=True, exist_ok=True)
    if not os.path.exists(DRAW_PATH + f'/draw_card/{path}/{codename}.png'):
        try:
            async with aiohttp.ClientSession(headers=headers) as session:
                async with session.get(url, timeout=7) as response:
                    async with aiofiles.open(DRAW_PATH + f'/draw_card/{path}/{codename}.png', 'wb') as f:
                        await f.write(await response.read())
                        print(f'下载 {path_dict[path]} 图片成功，名称：{name}，url：{url}')
                        return True
        except TimeoutError:
            print(f'下载 {path_dict[path]} 图片超时，名称：{name}，url：{url}')
            return False
        except InvalidURL:
            print(f'下载 {path_dict[path]} 链接错误，名称：{name}，url：{url}')
            return False
    else:
        # logger.info(f'{path_dict[path]} 图片 {name} 已存在')
        return False


@driver.on_startup
def _check_dir():
    for dir_name in path_dict.keys():
        _p = Path(DRAW_PATH + f'/draw_card/' + dir_name)
        if not _p.exists():
            _p.mkdir(parents=True, exist_ok=True)


async def generate_img(card_set: Union[Set, List], game_name: str, star_list: list, color_role_star_list: list = None) -> str:
    # try:
    img_list = []
    color_list = []
    if color_role_star_list:
        SIX_LIST = color_role_star_list[0]
        FIVE_LIST = color_role_star_list[1]
        FOUR_LIST = color_role_star_list[2]
    for name in card_set:
        if game_name == 'prts':
            if name in SIX_LIST:
                color_list.append('#FFD700')
            elif name in FIVE_LIST:
                color_list.append('#DAA520')
            elif name in FOUR_LIST:
                color_list.append('#9370D8')
            else:
                color_list.append('white')
        pyname = cn2py(name)
        img_list.append(DRAW_PATH + f'/draw_card/{game_name}/{pyname}.png')
    img_len = len(img_list)
    w = 100 * 10
    if img_len <= 10:
        w = 100 * img_len
        h = 100
    elif img_len % 10 == 0:
        h = 100 * int(img_len / 10)
    else:
        h = 100 * int(img_len / 10) + 100
    card_img = await asyncio.get_event_loop().run_in_executor(None, _pst, h, img_list, game_name, color_list)
    num = 0
    for n in star_list:
        num += n
    A = CreateImg(w, h)
    A.paste(card_img)
    return A.pic2bs4()


def _pst(h: int, img_list: list, game_name: str, color_list: list):
    card_img = CreateImg(100 * 10, h, 100, 100)
    idx = 0
    for img in img_list:
        try:
            if game_name == 'prts':
                bk = CreateImg(100, 100, color=color_list[idx])
                b = CreateImg(94, 94, background=img)
                bk.paste(b, (3, 3))
                b = bk
                idx += 1
            else:
                b = CreateImg(100, 100, background=img)
        except FileNotFoundError:
            print(f'{img} not exists')
            b = CreateImg(100, 100, color='black')
        card_img.paste(b)
    return card_img


def init_star_rst(star_list: list, cnlist: list, max_star_list: list, max_star_olist: list, up_list: list = None) -> str:
    rst = ''
    for i in range(len(star_list)):
        if star_list[i]:
            rst += f'[{cnlist[i]}×{star_list[i]}] '
    rst += '\n'
    for i in range(len(max_star_list)):
        if max_star_list[i] in up_list:
            rst += f'第 {max_star_olist[i]+1} 抽获取UP {max_star_list[i]}\n'
        else:
            rst += f'第 {max_star_olist[i]+1} 抽获取 {max_star_list[i]}\n'
    return rst


def max_card(_dict: dict):
    _max_value = max(_dict.values())
    _max_user = list(_dict.keys())[list(_dict.values()).index(_max_value)]
    return f'抽取到最多的是{_max_user}，共抽取了{_max_value}次'
    # ThreeHighest = nlargest(3, operator_dict, key=operator_dict.get)
    # rst = '最喜欢你的前三位是干员是：\n'
    # for name in ThreeHighest:
    #     rst += f'{name} 共投了 {operator_dict[name]} 份简历\n'
    # return rst[:-1]


def is_number(s) -> bool:
    try:
        float(s)
        return True
    except ValueError:
        pass
    try:
        import unicodedata
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass
    return False


def cn2py(word) -> str:
    temp = ""
    for i in pypinyin.pinyin(word, style=pypinyin.NORMAL):
        temp += ''.join(i)
    return temp







import ujson as json
import os
from nonebot.adapters.cqhttp import MessageSegment
import nonebot
import random
from .update_game_info import update_info
from .util import generate_img, init_star_rst
from .config import GENSHIN_FIVE_P, GENSHIN_G_FIVE_P, GENSHIN_FOUR_P, I72_ADD, DRAW_PATH

driver: nonebot.Driver = nonebot.get_driver()

genshin_dict = {}
genshin_arm_dict = {}
FIVE_C_LIST = []
FIVE_ARMS_LIST = []
FOUR_C_LIST = []
FOUR_ARMS_LIST = []
THREE_ARMS_LIST = []

genshin_five = {}
genshin_count = {}
genshin_pl_count = {}


async def genshin_draw(user_id: int, count: int):
    #                   0      1      2
    cnlist = ['★★★★★', '★★★★', '★★★']
    genshin_list, five_list, five_olist, five_dict, star_list = _format_card_information(count, user_id)
    rst = init_star_rst(star_list, cnlist, five_list, five_olist)
    print(five_list)
    temp = ''
    if count > 90:
        genshin_list = set(genshin_list)
    return MessageSegment.image("base64://" + await generate_img(genshin_list, 'genshin', star_list)) + '\n' + rst[:-1] + \
           temp[:-1] + f'\n距离保底发还剩 {90 - genshin_count[user_id] if genshin_count.get(user_id) else "^"} 抽' \
           + "\n【五星：0.6%，四星：5.1%\n第72抽开始五星概率每抽加0.585%】"


async def update_genshin_info():
    global genshin_dict, genshin_arm_dict, FIVE_ARMS_LIST, FIVE_C_LIST, FOUR_ARMS_LIST, FOUR_C_LIST, THREE_ARMS_LIST
    url = 'https://wiki.biligame.com/ys/角色筛选'
    data, code = await update_info(url, 'genshin')
    if code == 200:
        genshin_dict = data
        FIVE_C_LIST = [genshin_dict[name]['名称'] for name in genshin_dict if
                       genshin_dict[name]['稀有度'] == '5星' and genshin_dict[name]['名称'] != '旅行者']
        FOUR_C_LIST = [genshin_dict[name]['名称'] for name in genshin_dict if genshin_dict[name]['稀有度'] == '4星']
    url = 'https://wiki.biligame.com/ys/武器图鉴'
    data, code = await update_info(url, 'genshin_arm', ['头像', '名称', '类型', '稀有度.alt', '初始基础属性1',
                                                        '初始基础属性2', '攻击力（MAX）', '副属性（MAX）', '技能'])
    if code == 200:
        genshin_arm_dict = data
        FIVE_ARMS_LIST = [genshin_arm_dict[name]['名称'] for name in genshin_arm_dict if
                          genshin_arm_dict[name]['稀有度'] == '5星']
        FOUR_ARMS_LIST = [genshin_arm_dict[name]['名称'] for name in genshin_arm_dict if
                          genshin_arm_dict[name]['稀有度'] == '4星']
        THREE_ARMS_LIST = [genshin_arm_dict[name]['名称'] for name in genshin_arm_dict if
                           genshin_arm_dict[name]['稀有度'] == '3星']


# asyncio.get_event_loop().run_until_complete(update_genshin_info())


@driver.on_startup
async def init_data():
    global genshin_dict, genshin_arm_dict, FIVE_ARMS_LIST, FIVE_C_LIST, FOUR_ARMS_LIST, FOUR_C_LIST, THREE_ARMS_LIST
    if not os.path.exists(DRAW_PATH + 'genshin.json') or not os.path.exists(DRAW_PATH + 'genshin_arm.json'):
        await update_genshin_info()
    else:
        with open(DRAW_PATH + 'genshin.json', 'r', encoding='utf8') as f:
            genshin_dict = json.load(f)
        with open(DRAW_PATH + 'genshin_arm.json', 'r', encoding='utf8') as f:
            genshin_arm_dict = json.load(f)
        FIVE_C_LIST = [genshin_dict[name]['名称'] for name in genshin_dict if
                       genshin_dict[name]['稀有度'] == '5星' and genshin_dict[name]['名称'] != '旅行者']
        FIVE_ARMS_LIST = [genshin_arm_dict[name]['名称'] for name in genshin_arm_dict if
                          genshin_arm_dict[name]['稀有度'] == '5星']
        FOUR_C_LIST = [genshin_dict[name]['名称'] for name in genshin_dict if genshin_dict[name]['稀有度'] == '4星']
        FOUR_ARMS_LIST = [genshin_arm_dict[name]['名称'] for name in genshin_arm_dict if
                          genshin_arm_dict[name]['稀有度'] == '4星']
        THREE_ARMS_LIST = [genshin_arm_dict[name]['名称'] for name in genshin_arm_dict if
                           genshin_arm_dict[name]['稀有度'] == '3星']


# 抽取卡池
def _get_genshin_card(mode: int = 1, add: float = 0.0):
    rand = random.random()
    # 普通抽
    if mode == 1:
        if rand <= GENSHIN_FIVE_P + add:
            if random.random() < 0.5:
                return random.choice(FIVE_C_LIST), 0
            else:
                return random.choice(FIVE_ARMS_LIST), 0
        elif rand <= GENSHIN_FOUR_P:
            return random.choice(FOUR_C_LIST + FOUR_ARMS_LIST), 1
        else:
            return random.choice(THREE_ARMS_LIST), 2
    # 十连保底抽
    elif mode == 2:
        if rand <= GENSHIN_G_FIVE_P + add:
            if random.random() < 0.5:
                return random.choice(FIVE_C_LIST), 0
            else:
                return random.choice(FIVE_ARMS_LIST), 0
        else:
            return random.choice(FOUR_C_LIST + FOUR_ARMS_LIST), 1
    # 大保底
    elif mode == 3:
        if random.random() < 0.5:
            return random.choice(FIVE_C_LIST), 0
        else:
            return random.choice(FIVE_ARMS_LIST), 0


def _format_card_information(_count: int, user_id):
    genshin_list = []
    star_list = [0, 0, 0]
    five_olist = []
    five_list = []
    five_dict = {}
    add = 0.0
    if genshin_count.get(user_id) and _count <= 90:
        f_count = genshin_count[user_id]
    else:
        f_count = 0
    if genshin_pl_count.get(user_id) and _count <= 90:
        count = genshin_pl_count[user_id]
    else:
        count = 0
    for i in range(_count):
        count += 1
        f_count += 1
        # 十连保底
        if count == 10 and f_count != 90:
            if f_count >= 72:
                add += I72_ADD
            name, code = _get_genshin_card(2, add)
            count = 0
        # 大保底
        elif f_count == 90:
            name, code = _get_genshin_card(3)
        else:
            if f_count >= 72:
                add += I72_ADD
            name, code = _get_genshin_card(add=add)
            if code == 1:
                count = 0
        star_list[code] += 1
        if code == 0:
            if _count <= 90:
                genshin_five[user_id] = f_count
            add = 0.0
            f_count = 0
            five_list.append(name)
            five_olist.append(i)
            try:
                five_dict[name] += 1
            except KeyError:
                five_dict[name] = 1
        genshin_list.append(name)
    if _count <= 90:
        genshin_count[user_id] = f_count
        genshin_pl_count[user_id] = count
    return genshin_list, five_list, five_olist, five_dict, star_list


def reset_count(user_id: int):
    genshin_count[user_id] = 0
    genshin_pl_count[user_id] = 0

